import { css, LitElement } from 'lit';
import { property } from 'lit/decorators.js';
import { UUIEvent } from '@umbraco-ui/uui-base/lib/events';
import { defineElement } from '@umbraco-ui/uui-base/lib/registration';

class UUIIconHost {
  constructor(svg) {
    this.promise = new Promise((resolveMethod, rejectMethod) => {
      this.resolve = resolveMethod;
      this.reject = rejectMethod;
    });
    if (svg) {
      this.resolve(svg);
    }
  }
  set svg(svg) {
    this.resolve(svg);
  }
}

class UUIIconRegistryEvent extends UUIEvent {
  acceptRequest() {
    this.preventDefault();
  }
}
UUIIconRegistryEvent.ICON_REQUEST = "icon_request";

class UUIIconRegistry extends EventTarget {
  constructor() {
    super(...arguments);
    this.icons = {};
  }
  defineIcon(iconName, svgString) {
    if (this.icons[iconName]) {
      this.icons[iconName].svg = svgString;
    }
    this.icons[iconName] = new UUIIconHost(svgString);
  }
  async getIcon(iconName) {
    if (this.icons[iconName]) {
      return this.icons[iconName].promise;
    } else {
      const icon = new UUIIconHost();
      this.icons[iconName] = icon;
      const event = new UUIIconRegistryEvent(UUIIconRegistryEvent.ICON_REQUEST, {
        cancelable: true,
        detail: { iconName }
      });
      this.dispatchEvent(event);
      if (event.defaultPrevented === false) {
        console.log("reject");
        icon.reject();
      }
      return icon.promise;
    }
  }
}
const iconRegistry = new UUIIconRegistry();

var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
typeof require !== "undefined" ? require : (x) => {
  throw new Error('Dynamic require of "' + x + '" is not supported');
};
var __decorateClass = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result)
    __defProp(target, key, result);
  return result;
};
class UUIIconElement extends LitElement {
  constructor() {
    super(...arguments);
    this._name = null;
    this._svg = null;
    this.fallback = null;
  }
  get name() {
    return this._name;
  }
  set name(newValue) {
    this._name = newValue;
    if (this._name !== "" && this._name !== null) {
      iconRegistry.getIcon(this._name).then((svg) => {
        if (this.shadowRoot) {
          this.shadowRoot.innerHTML = svg;
        }
      }).catch(() => {
        if (this.fallback && this.shadowRoot) {
          this.shadowRoot.innerHTML = this.fallback;
        }
      });
    }
  }
  get svg() {
    return null;
  }
  set svg(newValue) {
    this._svg = newValue;
    if (this.shadowRoot) {
      this.shadowRoot.innerHTML = newValue || "";
    }
  }
  connectedCallback() {
    super.connectedCallback();
    if (this._svg) {
      this.shadowRoot.innerHTML = this._svg;
    }
  }
}
UUIIconElement.styles = [
  css`
      :host {
        display: inline-block;
        vertical-align: bottom;
        width: 1.15em;
        height: 1.15em;
      }

      :host svg {
        fill: currentColor;
      }
    `
];
__decorateClass([
  property()
], UUIIconElement.prototype, "name", 1);
__decorateClass([
  property()
], UUIIconElement.prototype, "svg", 1);
__decorateClass([
  property()
], UUIIconElement.prototype, "fallback", 2);

defineElement("uui-icon", UUIIconElement);
